//////////////////////////////////////////////////////////////////////////////
//
// A class to manage net threaded servers (TCP & TCPSSL)
// Juan M. Casillas
// assman@gsyc.inf.uc3m.es
//
// $Id: netthreadedserver.h,v 1.1 2001/07/21 14:34:49 assman Exp $
//
// $Log: netthreadedserver.h,v $
// Revision 1.1  2001/07/21 14:34:49  assman
// Added the netthreadedserver to the library tree
//
// Revision 1.1  2001/06/24 13:23:23  assman
// added new rules to compile all from a single makefile
// added support for new log types (syslog, file, dummy logger)
// added support for a generic network threaded server
//
//
//////////////////////////////////////////////////////////////////////////////

#ifndef __NETTHREADEDSERVER_H__
#define __NETTHREADEDSERVER_H__

#include <signals>
#include <threadedserver>
#include <netcomm/tcp>

#ifdef __LIBNETCOMM_USE_SSL__
#include <netcomm/tcpssl>
#endif

#include <stack>
#include <lock>
#include <sem>

class NetThreadedServer: public GenericThreadServer {
 public:

    static const int TCP;
    static const int TCPSSL;

    NetThreadedServer(int maxth, int block, int sec_timeout=60, 
		      int msec_timeout=0, int reuse_socket=1, int
		      socket_queue=5, int type= TCP):
	GenericThreadServer(maxth,block) {
	if (type == TCP) 
	    server = new TCP_Connection_Adapter_Server(sec_timeout,
						       msec_timeout,
						       reuse_socket,
						       socket_queue);

	#ifdef __LIBNETCOMM_USE_SSL__
	if (type == TCPSSL) 
	    server = new TCPSSL_Connection_Adapter_Server(sec_timeout,
							  msec_timeout,
							  reuse_socket,
							  socket_queue);
        #endif

	TOOLS::ignore_sigpipe();
	_servertype = type;
	//_th_avail = maxth; // all threads are available at start
	//_pth_lock.lock(); // lock at the beginning
	// _incoming_lock.lock();

    };

    virtual ~NetThreadedServer() { delete(server);  }

    virtual void *_dispatchthread(void) = 0;
    virtual void _workerdispatchthread(void) = 0;

    int servertype(void) { return(_servertype); }

    Connection_Adapter_Server         *server;

 protected:

    int _servertype;
    vector<Connection_Handler *> _incoming_data;
    Semaphore                   _th_sync;
    Lock                          _incoming_lock;
    int                         _th_avail;
    Lock                        _work_lock;

    //LockCond                    _th_avail_lock;
    //Lock                        _pth_lock;


};

#endif
