//////////////////////////////////////////////////////////////////////////////
//
// $Id: threadedserver.h,v 1.1 2001/07/21 14:34:56 assman Exp $
// A generic thread server
//
//////////////////////////////////////////////////////////////////////////////
//
// 15/May/2001 (Tue 11:33) This server first create all the threads, then
//                         put them waitting to a condition (this are under
//                         control of the workerdispatchthread and 
//                         dispatchthread
//////////////////////////////////////////////////////////////////////////////
//
// Requires lock include file to proper work
//
// $Log: threadedserver.h,v $
// Revision 1.1  2001/07/21 14:34:56  assman
// Added the threadedserver to the library tree
//
// Revision 1.3  2001/06/24 13:43:29  assman
// added full cvs tags
//
//
//////////////////////////////////////////////////////////////////////////////

#ifndef __GENERIC_THREAD_SERVER_H__
#define __GENERIC_THREAD_SERVER_H__

#include <vector>
#include <unistd.h>
#include <pthread.h>

class GenericThreadServer {
 public:

  GenericThreadServer(int maxth, int block):
      _pth_max_num(maxth),
      _pth_num(0),
      _block(block),
      _threads_created(false) {

    int res;

    //
    // thread attribute init
    //

    pthread_attr_init(&_detached);
    res = pthread_attr_setdetachstate(&_detached,PTHREAD_CREATE_DETACHED);
    if (res != 0) {
      cout << "can't set the detached state" << endl;
      exit(-1);
    }
  }
  
  virtual ~GenericThreadServer() {
    destroy();
  }

  void destroy(void) {
    pthread_attr_destroy(&_detached);
    if (_threads_created) 
	for (int i=0; i< _pth_max_num ; i++) 
	    pthread_cancel(_pthreads[i]);
  }


  long max_threads(void) { return(_pth_max_num);  }
  long used_threads(void) { return(_pth_num); }


  //
  // begin serve threads ...
  // if not blocked, create a new threads for the dispatcher,
  // else if non blocking beheavior has been selected, this call
  // will block until workerdispatchthread will end
  //

  void dispatch(void) {
   
    if (!_block) {
      int res;
      
      res = pthread_create(&_pthdispatch, &_detached, 
			   workerdispatchthread, (void *) this);
      if (res != 0) {
	cout << "can't create thread" << endl;
	exit(-1);
      }  
    }
    else {
      _workerdispatchthread();
    }
  }


  void createthreads(void) {
    int n;
    int val;
    pthread_t pid;
    int res;

    val = _pth_max_num;

    for (n=0; n< val; n++) {
      res = pthread_create(&pid, &_detached, dispatchthread, (void *) this);
      if (res != 0) {
	cout << "can't create thread" << endl;
	exit(-1);
      }
      //
      // add into the thread id container
      //
   
      _pthreads.push_back(pid);
    } 
    _threads_created = true;
  }

  //
  // this function is used to be called by
  // thread handlers. The real function
  // called is _dispatchtread
  //

  static void *dispatchthread(void *data) {
    GenericThreadServer *s;
    s = (GenericThreadServer *)data;
    s->_dispatchthread();
    return(NULL);
  }

  //
  // thread that do the work
  // this is the main thread (by example, the thread for a receiver)
  // accepts a pointer for this object, and launch the real worker
  // dispatchthread. is used to allow register the handler for the
  // thread
  //

  static void *workerdispatchthread(void *data) {
    GenericThreadServer *s;
    s = (GenericThreadServer *)data;
    s->_workerdispatchthread();
    return(NULL);
  }

  //
  // virtual methods this methods have to been defined in the
  // implementation
  //

  virtual void *_dispatchthread(void) = 0;
  virtual void _workerdispatchthread(void) = 0;

  static const int blocking; 
  static const int nonblocking;

protected:

    vector<pthread_t>       _pthreads;     // threads ids
    pthread_t               _pthdispatch;  // dispatch thread
    pthread_attr_t          _detached;     // thread attribute

    long                    _pth_max_num;  // max thread number
    int                     _pth_num;      // thread number
    int                     _block;        // is blocking ??
    bool                    _threads_created;  // if we have started the thr
};


#endif


